package com.uoc.textiller.controller;

import java.util.List;

import javax.validation.Valid;

import com.uoc.textiller.model.Fornitura;
import com.uoc.textiller.service.ProveidorService;
import com.uoc.textiller.service.FornituraService;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;

/**
 * Controlador Fornitura
 * @author lrodriguezmos@uoc
 */
@Controller
@RequestMapping("/fornitures")
public class FornituraController {

    private static final String FORNITURA_ATTR = "fornitura";
    private static final String FORNITURESLIST_ATTR = "fornituresList";
    private static final String FORNITURES_VIEW = "fornitures";
    private static final String FORNITURA_VIEW = "fornitura";
    private static final String FORNITURAFORM_VIEW = "fornituraForm";
    private static final String FORNITURA_REDIRECT = "redirect:/fornitures/consultar?id=";
    private static final String FORNITURES_REDIRECT = "redirect:/fornitures/llista";
    
    @Autowired
    private FornituraService fornituraService;

    @Autowired
    private ProveidorService proveidorService;


    @GetMapping({"","/"})
    public String fornituresRedirect() {
        
        return FORNITURES_REDIRECT;
    }

    @GetMapping("/llista")
    public String fornituresList(Model model, @RequestParam(name = "idProveidor", required = false) Long idProveidor) {
        
        if(idProveidor != null) {
            List<Fornitura> fornituresList = fornituraService.getFornituresListByProveidor(idProveidor);
            model.addAttribute(FORNITURESLIST_ATTR, fornituresList);
            model.addAttribute("fornituresSubtitol", proveidorService.findById(idProveidor).getNom());
            return FORNITURES_VIEW;
        } else {
            List<Fornitura> fornituresList = fornituraService.getFornituresList();
            model.addAttribute(FORNITURESLIST_ATTR, fornituresList);
            return findPaginated(1, model);
        }
    }

    @GetMapping("/pag/{pageNo}")
    public String findPaginated(@PathVariable(value = "pageNo") int pageNo, Model model) {
    
        int pageSize = 10;
    
        Page<Fornitura> page = fornituraService.findPaginated(pageNo, pageSize);
        List<Fornitura> fornituresList = page.getContent();

        model.addAttribute("view", FORNITURES_VIEW);
        model.addAttribute("currentPage", pageNo);
        model.addAttribute("totalPages", page.getTotalPages());
        model.addAttribute("totalItems", page.getTotalElements());
        model.addAttribute(FORNITURESLIST_ATTR, fornituresList);
        return FORNITURES_VIEW;
    }

    @GetMapping("/cercar")
    public String search(Model model, String keyword) {
        if(keyword!=null) {
            List<Fornitura> fornituresList = fornituraService.findByKeyword(keyword);
            model.addAttribute(FORNITURESLIST_ATTR, fornituresList);
            model.addAttribute("fornituresSubtitol", "Paraula clau: "+keyword);
        } else {
            List<Fornitura> fornituresList = fornituraService.getFornituresList();
            model.addAttribute(FORNITURESLIST_ATTR, fornituresList);
        }
        
        return FORNITURES_VIEW;
    }

    @GetMapping("/consultar")
    public String fornituraView(@RequestParam(name = "id", required = true) Long id, Model model) {

        Fornitura fornitura = fornituraService.findById(id);

        model.addAttribute(FORNITURA_ATTR, fornitura);

        return FORNITURA_VIEW;
    }
    
    @GetMapping("/formulari")
    public String fornituraForm(@RequestParam(name = "id", required = true) Long id, 
                                @RequestParam(name = "idProveidor", required = false) Long idProveidor,
                                Model model) {

        Fornitura fornitura = new Fornitura();
        
        model.addAttribute("proveidorsList", proveidorService.getProveidorsList());

        if(id > 0){
            fornitura = fornituraService.findById(id);
            model.addAttribute("titol", "Modificar fornitura");
            model.addAttribute("subtitol", "");
        } else {
            model.addAttribute("titol", "Afegir fornitura");
            model.addAttribute("subtitol", "Introdueix les dades de la nova fornitura:");
            if(idProveidor != null) {
                // quan s'afegeix la fornitura des del proveïdor
                fornitura.setProveidor(proveidorService.findById(idProveidor));
            }
        }

        model.addAttribute(FORNITURA_ATTR, fornitura);

        return FORNITURAFORM_VIEW;
    }
    
    @PostMapping("/guardar")
    public String save(@RequestParam(name = "nom_anterior", required = true) String nom_anterior, 
                        @Valid Fornitura fornitura, BindingResult result, Model model) {

        //en cas que modifiqui el nom (atribut únic a la base dades), verifiquem que no existeixi ja
        if(!fornitura.getNom().equals(nom_anterior) && fornituraService.existsNom(fornitura.getNom())) {
            result.rejectValue("nom", "error.nom", "Aquest nom ja existeix a la base de dades");
            fornitura.setNom(nom_anterior);
        }

        if(result!=null && result.hasErrors()) {
        
            model.addAttribute("proveidorsList", proveidorService.getProveidorsList());

            if(fornitura.getId()!=null && fornitura.getId() != 0) {
                model.addAttribute("titol", "Modificar fornitura");
                model.addAttribute("subtitol", "");
            } else {
                model.addAttribute("titol", "Afegir fornitura");
                model.addAttribute("subtitol", "Introdueix les dades de la nova fornitura:");
            }
            
            return FORNITURAFORM_VIEW;
        } 
        
        Fornitura f = fornituraService.save(fornitura);
        model.addAttribute(FORNITURA_ATTR, f);

        return FORNITURA_REDIRECT+f.getId();
    }
    
    @GetMapping("/eliminar")
    public String remove(@RequestParam(name = "id", required = true) Long id, Model model) {
        
        //no podem esborrar la fornitura si té escandalls associats
        if(fornituraService.hasEscandalls(id)){
            model.addAttribute("errorMsg", 
                "Aquesta fornitura té escandalls associats.<br>"+
                "Si vols eliminar la fornitura, primer hauràs d'eliminar els escandalls que la inclouen.");

        } else {
            fornituraService.remove(id);
            model.addAttribute("successMsg", "S'ha eliminat la fornitura");
        }

        List<Fornitura> fornituresList = fornituraService.getFornituresList();
        model.addAttribute(FORNITURESLIST_ATTR, fornituresList);

        return FORNITURES_VIEW;
    }
}
