package com.uoc.textiller.controller;

import java.util.List;

import javax.validation.Valid;

import com.uoc.textiller.model.Material;
import com.uoc.textiller.service.MaterialService;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;

/**
 * Controlador Material
 * @author lrodriguezmos@uoc
 */
@Controller
@RequestMapping("/materials")
public class MaterialController {

    private static final String MATERIAL_ATTR = "material";
    private static final String MATERIALSLIST_ATTR = "materialsList";
    private static final String MATERIALS_VIEW = "materials";
    private static final String MATERIALFORM_VIEW = "materialForm";
    private static final String MATERIALS_REDIRECT = "redirect:/materials/llista";

    @Autowired
    private MaterialService materialService;

    @GetMapping({"","/"})
    public String materialsRedirect() {
        
        return MATERIALS_REDIRECT;
    }

    @GetMapping("/llista")
    public String materialsList(Model model) {
        
        List<Material> materialsList = materialService.getMaterialsList();
        
        model.addAttribute(MATERIAL_ATTR, materialsList);

        return findPaginated(1, model);
    }

    @GetMapping("/pag/{pageNo}")
    public String findPaginated(@PathVariable(value = "pageNo") int pageNo, Model model) {
    
        int pageSize = 10;
    
        Page<Material> page = materialService.findPaginated(pageNo, pageSize);
        List<Material> usuarisList = page.getContent();

        model.addAttribute("view", MATERIALS_VIEW);
        model.addAttribute("currentPage", pageNo);
        model.addAttribute("totalPages", page.getTotalPages());
        model.addAttribute("totalItems", page.getTotalElements());
        model.addAttribute(MATERIALSLIST_ATTR, usuarisList);
        return MATERIALS_VIEW;
    }

    @GetMapping("/cercar")
    public String search(Model model, String keyword) {
        if(keyword!=null) {
            List<Material> materialsList = materialService.findByKeyword(keyword);
            model.addAttribute(MATERIALSLIST_ATTR, materialsList);
        } else {
            List<Material> materialsList = materialService.getMaterialsList();
            model.addAttribute(MATERIALSLIST_ATTR, materialsList);
        }
        
        return MATERIALS_VIEW;
    }
    
    @GetMapping({"/formulari"})
    public String materialForm(@RequestParam(name = "id", required = true) Long id, Model model) {

        Material material = new Material();

        if(id > 0){
            material = materialService.findById(id);
            model.addAttribute("titol", "Modificar material");
            model.addAttribute("subtitol", "");
        } else {
            model.addAttribute("titol", "Afegir material");
            model.addAttribute("subtitol", "Introdueix el nom del nou material:");
        }

        model.addAttribute(MATERIAL_ATTR, material);

        return MATERIALFORM_VIEW;
    }
    
    @PostMapping("/guardar")
    public String save(@RequestParam(name = "nom_anterior", required = true) String nom_anterior, 
                        @Valid Material material, BindingResult result, Model model) {

        //en cas que modifiqui el nom (atribut únic a la base dades), verifiquem que no existeixi ja
        if(!material.getNom().equals(nom_anterior) && materialService.existsNom(material.getNom())) {
            result.rejectValue("nom", "error.nom", "Aquest nom ja existeix a la base de dades");
            material.setNom(nom_anterior);
        }

        if(result!=null && result.hasErrors()) {
            if(material.getId()!=null && material.getId() != 0) {
                model.addAttribute("titol", "Modificar material");
                model.addAttribute("subtitol", "");
            } else {
                model.addAttribute("titol", "Afegir usuari");
                model.addAttribute("subtitol", "Introdueix el nom del nou material:");
            }
            return MATERIALFORM_VIEW;
        } 
        
        model.addAttribute(MATERIAL_ATTR, materialService.save(material));
        return MATERIALS_REDIRECT;
    }
    
    @GetMapping("/eliminar")
    public String remove(@RequestParam(name = "id", required = true) Long id, Model model) {
        
         //no podem esborrar el material si té composicions associades
        if(materialService.hasComposicions(id)){
            model.addAttribute("errorMsg", 
                "Aquest material té composicions associades.<br> "+
                "Si vols eliminar el material, primer hauràs <br> "+
                "d'eliminar les composicions de teixit que el contenen.");

        } else {
            materialService.remove(id);
            model.addAttribute("successMsg", "S'ha eliminat el material");
        }
        

        List<Material> materialsList = materialService.getMaterialsList();
        model.addAttribute(MATERIALSLIST_ATTR, materialsList);

        return MATERIALS_VIEW;
    }
    
}
