package com.uoc.textiller.controller;

import java.util.List;

import javax.validation.Valid;

import com.uoc.textiller.model.Usuari;
import com.uoc.textiller.service.UsuariService;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;

/**
 * Controlador Usuari
 * @author lrodriguezmos@uoc
 */
@Controller
@RequestMapping("/usuaris")
public class UsuariController {

    private static final String USUARI_ATTR = "usuari";
    private static final String USUARISLIST_ATTR = "usuarisList";
    private static final String USUARIS_VIEW = "usuaris";
    private static final String USUARIFORM_VIEW = "usuariForm";
    private static final String USUARIS_REDIRECT = "redirect:/usuaris/llista";
    
    @Autowired
    private UsuariService usuariService;


    @GetMapping({"","/"})
    public String usuarisRedirect() {
        
        return USUARIS_REDIRECT;
    }

    @GetMapping("/llista")
    public String usuarisList(Model model) {

        List<Usuari> usuarisList = usuariService.getUsuarisList();

        model.addAttribute(USUARISLIST_ATTR, usuarisList);

        return findPaginated(1, model);
    }

    @GetMapping("/pag/{pageNo}")
    public String findPaginated(@PathVariable(value = "pageNo") int pageNo, Model model) {
    
        int pageSize = 10;
    
        Page<Usuari> page = usuariService.findPaginated(pageNo, pageSize);
        List<Usuari> usuarisList = page.getContent();

        model.addAttribute("view", USUARIS_VIEW);
        model.addAttribute("currentPage", pageNo);
        model.addAttribute("totalPages", page.getTotalPages());
        model.addAttribute("totalItems", page.getTotalElements());
        model.addAttribute(USUARISLIST_ATTR, usuarisList);
        return USUARIS_VIEW;
    }

    @GetMapping("/cercar")
    public String search(Model model, String keyword) {
        if(keyword!=null) {
            List<Usuari> usuarisList = usuariService.findByKeyword(keyword);
            model.addAttribute(USUARISLIST_ATTR, usuarisList);
        } else {
            List<Usuari> usuarisList = usuariService.getUsuarisList();
            model.addAttribute(USUARISLIST_ATTR, usuarisList);
        }
        
        return USUARIS_VIEW;
    }
    
    @GetMapping({"/formulari"})
    public String usuariForm(@RequestParam(name = "id", required = true) Long id, Model model) {

        Usuari usuari = new Usuari();

        if(id > 0){
            usuari = usuariService.findById(id);
            model.addAttribute("titol", "Modificar usuari");
            model.addAttribute("subtitol", "");
        } else {
            model.addAttribute("titol", "Afegir usuari");
            model.addAttribute("subtitol", "Introdueix les dades del nou usuari:");
        }

        model.addAttribute(USUARI_ATTR, usuari);

        return USUARIFORM_VIEW;
    }
    
    @PostMapping("/guardar")
    public String save(@RequestParam(name = "nom_anterior", required = true) String nom_anterior, 
                        @Valid Usuari usuari, BindingResult result, Model model) {

        //en cas que modifiqui el nom (atribut únic a la base dades), verifiquem que no existeixi ja
        if(!usuari.getNom().equals(nom_anterior) && usuariService.existsNom(usuari.getNom())) {
            result.rejectValue("nom", "error.nom", "Aquest nom ja existeix a la base de dades");
            usuari.setNom(nom_anterior);
        }

        if(result!=null && result.hasErrors()) {
            if(usuari.getId()!=null && usuari.getId() != 0) {
                model.addAttribute("titol", "Modificar usuari");
                model.addAttribute("subtitol", "");
            } else {
                model.addAttribute("titol", "Afegir usuari");
                model.addAttribute("subtitol", "Introdueix les dades del nou usuari:");
            }
            return USUARIFORM_VIEW;
        } 
        
        model.addAttribute(USUARI_ATTR, usuariService.save(usuari));
        return USUARIS_REDIRECT;
    }
    
    @GetMapping("/eliminar")
    public String remove(@RequestParam(name = "id", required = true) Long id, Model model) {
        
        Usuari u = usuariService.findById(id);

        //no es pot eliminar l'usuari per defecte "admin"
        if(u.getNom().equals("admin")) {
            model.addAttribute("errorMsg", "No es pot eliminar l'usuari admin");
        } else {
            //no podem esborrar l'usuari si té escandalls associats
            if(usuariService.hasEscandalls(id)){
                model.addAttribute("errorMsg", 
                    "Aquest usuari té escandalls associats.<br>"+
                    "Si vols eliminar l'usuari, primer hauràs d'eliminar<br>"+ 
                    "o modificar l'usuari dels seus escandalls.");
    
            } else {
                usuariService.remove(id);
                model.addAttribute("successMsg", "S'ha eliminat l'usuari");
            }
        }

        List<Usuari> usuarisList = usuariService.getUsuarisList();
        model.addAttribute(USUARISLIST_ATTR, usuarisList);
        
        return USUARIS_VIEW;
    }
}
