package com.uoc.textiller.service;

import java.util.List;

import com.uoc.textiller.model.Escandall;
import com.uoc.textiller.model.EscandallFornitura;
import com.uoc.textiller.model.EscandallTeixit;
import com.uoc.textiller.model.EscandallProces;
import com.uoc.textiller.model.Usuari;
import com.uoc.textiller.repository.EscandallFornituraRepository;
import com.uoc.textiller.repository.EscandallRepository;
import com.uoc.textiller.repository.EscandallTeixitRepository;
import com.uoc.textiller.repository.EscandallProcesRepository;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;

/**
 * Implementació dels mètodes de FornituraService
 * @author lrodriguezmos@uoc
 */
@Service
public class EscandallServiceImpl implements EscandallService {

    @Autowired
    EscandallRepository escandallRepository;

    @Autowired
    EscandallTeixitRepository escandallTeixitRepository;

    @Autowired
    EscandallFornituraRepository escandallFornituraRepository;

    @Autowired
    EscandallProcesRepository escandallProcesRepository;

    public EscandallServiceImpl(EscandallRepository escandallRepository) {
        this.escandallRepository = escandallRepository;
    }

    /** Cerca per ID */
    @Override
    public Escandall findById(Long id) {
        return escandallRepository.findById(id).get();
    }

    /** Cerca per nom */
    @Override
    public Escandall findByNom(String nom) {
        return escandallRepository.findByNom(nom);
    }

    /** Llista d'escandalls */
    @Override
    public List<Escandall> getEscandallsList() {
        return escandallRepository.findAll();
    }

    /** Llista d'escandalls per usuari */
    @Override
    public List<Escandall> getEscandallsUsuariList(Usuari usuari) {
        return escandallRepository.findByUsuari(usuari);
    }

    /** Afegir o modificar escandall */
    @Override
    public Escandall save(Escandall escandall) {

        Escandall e;

        if(escandall.getId()!=null) { //si està modificant
            //usuari i data mantenen sempre el seu valor original
            e = findById(escandall.getId());
            escandall.setData(e.getData());
            escandall.setUsuari(e.getUsuari());
        }

        e = escandallRepository.save(escandall);

        if(e.getReferencia() == null && e.getId() > 0) {
            //referència autogenerada a partir de l'identificador
            String referencia = "E" + String.format("%07d", e.getId());
            e.setReferencia(referencia);
            return escandallRepository.save(e);
        }

        return e;
    }

    /** Eliminar escandall */
    @Override
    public void remove(Long id) {

        Escandall escandall = escandallRepository.getById(id);
        /** eliminem primer els teixits, fornitures i processos de l'escandall */
        for (EscandallTeixit teixit : escandall.getTeixits()) {
            removeTeixit(teixit.getId());
        }

        for (EscandallFornitura fornitura : escandall.getFornitures()) {
            removeFornitura(fornitura.getId());
        }

        for (EscandallProces proces : escandall.getProcessos()) {
            removeProces(proces.getId());
        }

        escandallRepository.deleteById(id);
        
    }

    /** Booleà que indica si ja existeix el nom de l'escandall */
    @Override
    public boolean existsNom(String nom) {
        return escandallRepository.findByNom(nom)!=null;
    }

    /** Cerca a la taula */
    @Override
    public List<Escandall> findByKeyword(String keyword) {
        return escandallRepository.findByKeyword(keyword);
    }

    /** Paginació de la taula */
    @Override
    public Page<Escandall> findPaginated(int pageNo, int pageSize) {
        Pageable pageable = PageRequest.of(pageNo - 1, pageSize);
        return this.escandallRepository.findAll(pageable);
    }

    /** Afegir un teixit a l'escandall */
    @Override
    public Escandall saveTeixit(Escandall escandall, EscandallTeixit escandallTeixit) {

        List<EscandallTeixit> teixits = escandall.getTeixits();
        teixits.add(escandallTeixit);
        escandall.setTeixits(teixits);

        escandallTeixit.setEscandall(escandall);
        escandallTeixitRepository.save(escandallTeixit);

        return escandallRepository.save(escandall);
    }

    /** Afegir una fornitura a l'escandall */
    @Override
    public Escandall saveFornitura(Escandall escandall, EscandallFornitura escandallFornitura) {

        List<EscandallFornitura> fornitures = escandall.getFornitures();
        fornitures.add(escandallFornitura);
        escandall.setFornitures(fornitures);

        escandallFornitura.setEscandall(escandall);
        escandallFornituraRepository.save(escandallFornitura);

        return escandallRepository.save(escandall);
    }

    /** Afegir un procés a l'escandall */
    @Override
    public Escandall saveProces(Escandall escandall, EscandallProces escandallProces) {

        List<EscandallProces> processos = escandall.getProcessos();
        processos.add(escandallProces);
        escandall.setProcessos(processos);

        escandallProces.setEscandall(escandall);
        escandallProcesRepository.save(escandallProces);

        return escandallRepository.save(escandall);
    }

    /** Booleà que indica si el teixit a afegir a l'escandall
     * ja existeix per aquell escandall */
    @Override
    public boolean hasTeixit(List<EscandallTeixit> teixitsEscandall, Long id_teixit) {
        
        for (EscandallTeixit et : teixitsEscandall) {
            if(et.getTeixit().getId() == id_teixit) {
                return true;
            }
        }
        return false;
    }   

    /** Booleà que indica si la fornitura a afegir a l'escandall
     * ja existeix per aquell escandall */
    @Override
    public boolean hasFornitura(List<EscandallFornitura> fornituresEscandall, Long id_fornitura) {
        
        for (EscandallFornitura ef : fornituresEscandall) {
            if(ef.getFornitura().getId() == id_fornitura) {
                return true;
            }
        }
        return false;
    }

    /** Booleà que indica si el procés a afegir a l'escandall
     * ja existeix per aquell escandall */
    @Override
    public boolean hasProces(List<EscandallProces> processosEscandall, Long id_proces) {
        
        for (EscandallProces ep : processosEscandall) {
            if(ep.getProces().getId() == id_proces) {
                return true;
            }
        }
        return false;
    }   

    /** Eliminar un teixit de l'escandall */
    @Override
    public void removeTeixit(Long id) {
        escandallTeixitRepository.deleteById(id);
        
    }

    /** Eliminar una fornitura de l'escandall */
    @Override
    public void removeFornitura(Long id) {
        escandallFornituraRepository.deleteById(id);
        
    }   

    /** Eliminar un procés de l'escandall */
    @Override
    public void removeProces(Long id) {
        escandallProcesRepository.deleteById(id);
        
    }
    
}
