package com.uoc.textiller;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotEquals;

import java.math.BigDecimal;
import java.util.List;
import java.util.Optional;

import com.uoc.textiller.model.Proveidor;
import com.uoc.textiller.model.Fornitura;
import com.uoc.textiller.repository.ProveidorRepository;
import com.uoc.textiller.repository.FornituraRepository;

import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.jdbc.AutoConfigureTestDatabase;
import org.springframework.boot.test.autoconfigure.jdbc.AutoConfigureTestDatabase.Replace;
import org.springframework.boot.test.autoconfigure.orm.jpa.DataJpaTest;

@DataJpaTest
@AutoConfigureTestDatabase(replace = Replace.NONE)
public class TestFornituraJPA {

    @Autowired
    private FornituraRepository repoFornitura;

    @Autowired
    private ProveidorRepository repoProveidor;

    public Proveidor defaultProveidor(){
        //necessitem un proveïdor per afegir fornitures
        Proveidor proveidor = new Proveidor();
        proveidor.setNom("Prov1");
        proveidor.setTelefon("123");
        proveidor.setEmail("email@email.com");
        return repoProveidor.save(proveidor);
    }

    /**TEST: Afegir dos fornitures diferents a la base de dades 
     * 
     * AVÍS: és necessari tenir la base de dades buida per a la correcta execució
    */
    @Test
    public void testAfegirFornitures() {

        Proveidor proveidor = defaultProveidor();

        Fornitura fornitura1 = new Fornitura();
        fornitura1.setNom("Fornitura1");
        fornitura1.setProveidor(proveidor);
        fornitura1.setCost(BigDecimal.valueOf(1));
        fornitura1.setEstoc(1);

        Fornitura fornitura2 = new Fornitura();
        fornitura2.setNom("Fornitura2");
        fornitura2.setProveidor(proveidor);
        fornitura2.setCost(BigDecimal.valueOf(2));
        fornitura2.setEstoc(2);

        repoFornitura.save(fornitura1);
        repoFornitura.save(fornitura2);

        repoFornitura.flush();

        //resultat esperat 1: la cerca per nom de "Fornitura1" hauria de retornar una fornitura vàlida
        assertNotEquals(null, repoFornitura.findByNom("Fornitura1"));
        //resultat esperat 2: la cerca per nom de "Fornitura2" hauria de retornar una fornitura vàlida
        assertNotEquals(null, repoFornitura.findByNom("Fornitura2"));
        //resultat esperat 3: el total de fornitures hauria de ser 2
        assertEquals(2, repoFornitura.findAll().size());

    }

    /**TEST: Editar una fornitura existent a la base de dades */
    @Test
    public void testEditarFornitura() {

        Proveidor proveidor = defaultProveidor();

        Fornitura fornitura1 = new Fornitura();
        fornitura1.setNom("Fornitura1");
        fornitura1.setProveidor(proveidor);
        fornitura1.setCost(BigDecimal.valueOf(1));
        fornitura1.setEstoc(1);

        Fornitura fornitura = repoFornitura.save(fornitura1);
        Long idFornitura = fornitura.getId();

        Optional<Fornitura> fornituraO = repoFornitura.findById(idFornitura);
        Fornitura fornituraEditar = fornituraO.get();
        fornituraEditar.setDescripcio("Això és una descripció que abans no tenia.");
        Fornitura fornituraEditat = repoFornitura.save(fornituraEditar);
               
        repoFornitura.flush();
        
        //resultat esperat: la Fornitura1 hauria de tenir una descripció
        assertEquals(true, fornituraEditat.getDescripcio().length()>0);

    }

    /**TEST: Eliminar un fornitura existent de la base de dades */
    @Test
    public void testEliminarFornitura() {

        Proveidor proveidor = defaultProveidor();

        Fornitura fornitura1 = new Fornitura();
        fornitura1.setNom("Fornitura1");
        fornitura1.setProveidor(proveidor);
        fornitura1.setCost(BigDecimal.valueOf(1));
        fornitura1.setEstoc(1);

        Fornitura fornituraEliminar = repoFornitura.save(fornitura1);

        Long idFornitura = fornituraEliminar.getId();

        repoFornitura.deleteById(idFornitura);
      
        repoFornitura.flush();

        //resultat esperat: cercar per nom el fornitura eliminat "Fornitura1" hauria de retornar null
        assertEquals(null, repoFornitura.findByNom("Fornitura1"));
    }

    

    /**TEST: Cercar fornitures a partir d'una paraula clau */
    @Test
    public void testCercarFornitures() {

        Proveidor proveidor = defaultProveidor();

        Fornitura fornitura1 = new Fornitura();
        fornitura1.setNom("Fornitura1");
        fornitura1.setProveidor(proveidor);
        fornitura1.setCost(BigDecimal.valueOf(1));
        fornitura1.setEstoc(1);

        Fornitura fornitura2 = new Fornitura();
        fornitura2.setNom("Fornitura2");
        fornitura2.setProveidor(proveidor);
        fornitura2.setCost(BigDecimal.valueOf(2));
        fornitura2.setEstoc(2);
        fornitura2.setDescripcio("Descripció");

        Fornitura fornitura3 = new Fornitura();
        fornitura3.setNom("Fornitura3");
        fornitura3.setProveidor(proveidor);
        fornitura3.setCost(BigDecimal.valueOf(2));
        fornitura3.setEstoc(2);
        fornitura3.setDescripcio("Descripció");

        repoFornitura.save(fornitura1);
        repoFornitura.save(fornitura2);
        repoFornitura.save(fornitura3);
      
        repoFornitura.flush();


        List<Fornitura> fornituraList = repoFornitura.findByKeyword("Fornitura");
        List<Fornitura> descripcioList = repoFornitura.findByKeyword("Descripció");

        //resultat esperat 1: cercar per "Fornitura" hauria de retornar una llista de 3 fornitures
        assertEquals(3, fornituraList.size());
        //resultat esperat 2: cercar per "Descripció" hauria de retornar una llista d'1 fornitura
        assertEquals(2, descripcioList.size());
    }
    
}
