package com.uoc.textiller;

import static org.junit.jupiter.api.Assertions.assertEquals;

import java.util.List;
import java.util.Optional;

import com.uoc.textiller.model.Usuari;
import com.uoc.textiller.model.enums.EstatsUsuari;
import com.uoc.textiller.model.enums.RolsUsuari;
import com.uoc.textiller.repository.UsuariRepository;

import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.jdbc.AutoConfigureTestDatabase;
import org.springframework.boot.test.autoconfigure.jdbc.AutoConfigureTestDatabase.Replace;
import org.springframework.boot.test.autoconfigure.orm.jpa.DataJpaTest;

@DataJpaTest
@AutoConfigureTestDatabase(replace = Replace.NONE)
public class TestUsuariJPA {

    @Autowired
    private UsuariRepository repoUsuari;

    /**TEST: Afegir dos usuaris diferents a la base de dades 
     * 
     * AVÍS: és necessari tenir la base de dades buida per a la correcta execució
    */
    @Test
    public void testAfegirUsuaris() {

        Usuari usuari1 = new Usuari();
        usuari1.setNom("John Admin");
        usuari1.setPassword("xxx");
        usuari1.setRol(RolsUsuari.ROLE_ADMINISTRADOR);
        usuari1.setEstat(EstatsUsuari.ACTIU);

        Usuari usuari2 = new Usuari();
        usuari2.setNom("Kalya Store");
        usuari2.setPassword("xxx");
        usuari2.setRol(RolsUsuari.ROLE_MAGATZEM);
        usuari2.setEstat(EstatsUsuari.ACTIU);

        repoUsuari.save(usuari1);
        repoUsuari.save(usuari2);

        repoUsuari.flush();

        Usuari usuariAdmin = repoUsuari.findByNom("John Admin");
        Usuari usuariMagatzem = repoUsuari.findByNom("Kalya Store");

        //resultat esperat 1: hauríem de tenir un usuari amb rol=ADMINISTRADOR
        assertEquals(RolsUsuari.ROLE_ADMINISTRADOR, usuariAdmin.getRol());
        //resultat esperat 2: hauríem de tenir un usuari amb rol=MAGATZEM
        assertEquals(RolsUsuari.ROLE_MAGATZEM, usuariMagatzem.getRol());
    }

    /**TEST: Editar un usuari existent a la base de dades */
    @Test
    public void testEditarUsuari() {

        Usuari usuari1 = new Usuari();
        usuari1.setNom("Mark Active");
        usuari1.setPassword("xxx");
        usuari1.setRol(RolsUsuari.ROLE_CONFECCIONISTA);
        usuari1.setEstat(EstatsUsuari.INACTIU);

        Usuari usuari = repoUsuari.save(usuari1);
        Long idUsuari = usuari.getId();

        Optional<Usuari> usuariO = repoUsuari.findById(idUsuari);
        Usuari usuariEditar = usuariO.get();
        usuariEditar.setEstat(EstatsUsuari.ACTIU);
        Usuari usuariEditat = repoUsuari.save(usuariEditar);
               
        repoUsuari.flush();
        
        //resultat esperat: l'usuari que abans era INACTIU hauria d'estar com a ACTIU
        assertEquals(EstatsUsuari.ACTIU, usuariEditat.getEstat());

    }

    /**TEST: Eliminar un usuari existent de la base de dades */
    @Test
    public void testEliminarUsuari() {
        
        Usuari usuari1 = new Usuari();
        usuari1.setNom("Mark Deleted");
        usuari1.setPassword("xxx");
        usuari1.setRol(RolsUsuari.ROLE_CONFECCIONISTA);
        usuari1.setEstat(EstatsUsuari.INACTIU);

        Usuari usuariEliminar = repoUsuari.save(usuari1);

        Long idUsuari = usuariEliminar.getId();

        repoUsuari.deleteById(idUsuari);
      
        repoUsuari.flush();

        //resultat esperat: cercar per nom l'usuari eliminat "Mark Deleted" hauria de retornar null
        assertEquals(null, repoUsuari.findByNom("Mark Deleted"));
    }

    

    /**TEST: Cercar usuaris a partir d'una' paraula clau */
    @Test
    public void testCercarUsuaris() {
        
        Usuari usuari1 = new Usuari();
        usuari1.setNom("Mark One");
        usuari1.setPassword("xxx");
        usuari1.setRol(RolsUsuari.ROLE_CONFECCIONISTA);
        usuari1.setEstat(EstatsUsuari.INACTIU);

        Usuari usuari2 = new Usuari();
        usuari2.setNom("Kalya One");
        usuari2.setPassword("xxx");
        usuari2.setRol(RolsUsuari.ROLE_MAGATZEM);
        usuari2.setEstat(EstatsUsuari.ACTIU);

        Usuari usuari3 = new Usuari();
        usuari3.setNom("John Two");
        usuari3.setPassword("xxx");
        usuari3.setRol(RolsUsuari.ROLE_MAGATZEM);
        usuari3.setEstat(EstatsUsuari.ACTIU);

        repoUsuari.save(usuari1);
        repoUsuari.save(usuari2);
        repoUsuari.save(usuari3);
      
        repoUsuari.flush();


        List<Usuari> usuarisOne = repoUsuari.findByKeyword("One");
        List<Usuari> usuarisMagatzem = repoUsuari.findByKeyword("Magatzem");
        List<Usuari> usuarisInactiu = repoUsuari.findByKeyword("Inactiu");

        //resultat esperat 1: cercar per "One" hauria de retornar una llista de 2 usuaris
        assertEquals(2, usuarisOne.size());
        //resultat esperat 2: cercar per "Magatzem" hauria de retornar una llista de 2 usuaris
        assertEquals(2, usuarisMagatzem.size());
        //resultat esperat 3: cercar per "Inactiu" hauria de retornar una llista d'1' usuaris
        assertEquals(1, usuarisInactiu.size());
    }
    
}
